# -*- coding: utf-8 -*-

from django.test import TestCase

from django.core.urlresolvers import reverse
from django.test.client import Client
from django.conf import settings

from django.contrib.auth.models import User

from secretquestions.models import Question, Answer


class ConfigurationTest(TestCase):

    client = Client()
    username = 'paul'
    password = 'lemay'

    def setUp(self):
        self.create_user()
        self.create_questions()

    def create_user(self):
        self.user = User.objects.create(username=self.username)
        self.user.set_password(self.password)
        self.user.save()

    def create_questions(self):
        self.question1 = Question.objects.create(text="question1")
        self.question2 = Question.objects.create(text="question2")
        self.question3 = Question.objects.create(text="question3")
        self.questions = (self.question1, self.question2, self.question3)

    def test_access_setup_questions_for_anonymous(self):
        url = reverse('sq_setup')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual('Location' in response, True)
        self.assertEqual(settings.LOGIN_URL in response['Location'], True)

    def test_access_setup_questions_for_authenticated(self):
        self.assertEqual(self.client.login(username=self.username,
            password=self.password), True)

        url = reverse('sq_setup')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_setting_answer_for_one_question(self):
        raw_password = 'xxx'
        self.assertEqual(self.client.login(username=self.username,
            password=self.password), True)
        url = reverse('sq_setup')

        data = {
                'form-TOTAL_FORMS': u'1',
                'form-INITIAL_FORMS': u'0',
                'form-MAX_NUM_FORMS': u'',
                'form-0-question': self.question1.id, 
                'form-0-secret': raw_password, 
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 302)
        answer = Answer.objects.get(user=self.user, question=self.question1)
        self.assertNotEqual(answer.secret, raw_password)
        self.assertNotEqual(answer.secret, '')


    def test_setting_empty_answer_for_one_question(self):
        raw_password = ''
        self.assertEqual(self.client.login(username=self.username,
            password=self.password), True)
        url = reverse('sq_setup')

        data = {
                'form-TOTAL_FORMS': u'1',
                'form-INITIAL_FORMS': u'0',
                'form-MAX_NUM_FORMS': u'',
                'form-0-question': self.question1.id, 
                'form-0-secret': raw_password, 
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        with self.assertRaises(Answer.DoesNotExist):
            Answer.objects.get(user=self.user, question=self.question1)

    def test_check_reset(self):
        raw_password = 'xxx'
        self.test_setting_answer_for_one_question()
        url = reverse('sq_setup')
        response = self.client.get(url)
        self.assertFalse(raw_password in response.content)
        answer = Answer.objects.get(user=self.user, question=self.question1)
        self.assertFalse(answer.secret in response.content)
        
